@@ -1,6 +1,6 @@
 #include "typedefTopo.h"
 #include "RvcgIO.h"
-#include <Rcpp.h>
+#include <RcppArmadillo.h>
 
 //using namespace vcg;
 using namespace tri;
