@@ -3,7 +3,7 @@
 
 #include "typedef.h"
 #include "RvcgIO.h" 
-#include <Rcpp.h>
+#include <RcppArmadillo.h>
 
 using namespace Rcpp;
 
