@@ -75,6 +75,7 @@ public:
 
   void start(MeshType &m, FacePointer p)
   {
+    tri::RequirePerFaceMark(m);
     mp=&m;
     while(!sf.empty()) sf.pop();
     UnMarkAll(m);
@@ -452,10 +453,11 @@ public:
     return count_removed;
   }
 
-  static void SplitSelectedVertexOnEdgeMesh(MeshType& m)
+  static int SplitSelectedVertexOnEdgeMesh(MeshType& m)
   {
     tri::RequireCompactness(m);
     tri::UpdateFlags<MeshType>::VertexClearV(m);
+    int count_split = 0;
     for(size_t i=0;i<m.edge.size();++i)
     {
       for(int j=0;j<2;++j)
@@ -464,11 +466,19 @@ public:
         if(vp->IsS())
         {
           if(!vp->IsV())
+	    {
             m.edge[i].V(j) = &*(tri::Allocator<MeshType>::AddVertex(m,vp->P()));
-          else vp->SetV();
+	    ++count_split;
+	    }
+          else 
+	    {
+	      vp->SetV();
+	    }
+	  
         }
       }
     }
+    return count_split;
   }
 
 
@@ -495,7 +505,7 @@ public:
     for(size_t i=0;i<m.vert.size();++i)
     {
       std::vector<VertexPointer> VVStarVec;
-      edge::VVStarVE<typename MeshType::EdgeType>(&(m.vert[i]),VVStarVec);
+      edge::VVStarVE(&(m.vert[i]),VVStarVec);
       if(VVStarVec.size()==2)
       {
         CoordType v0 = m.vert[i].P() - VVStarVec[0]->P();
@@ -975,6 +985,19 @@ public:
     }
     return nonManifoldCnt;
   }
+  /// Very simple test of water tightness. No boundary and no non manifold edges. 
+  /// Assume that it is orientable. 
+  /// It could be debated if a closed non orientable surface is watertight or not. 
+  /// 
+  /// The rationale of not testing orientability here is that 
+  /// it requires FFAdj while this test do not require any adjacency.
+  /// 
+  static bool IsWaterTight(MeshType & m)
+  {
+    int edgeNum=0,edgeBorderNum=0,edgeNonManifNum=0;
+    CountEdgeNum(m, edgeNum, edgeBorderNum,edgeNonManifNum);
+    return  (edgeBorderNum==0) && (edgeNonManifNum==0);
+  }
 
   static void CountEdgeNum( MeshType & m, int &total_e, int &boundary_e, int &non_manif_e )
   {
@@ -1692,8 +1715,7 @@ public:
     tri::Allocator<MeshType>::CompactVertexVector(m);
     typedef vcg::SpatialHashTable<VertexType, ScalarType> SampleSHT;
     SampleSHT sht;
-    tri::VertTmark<MeshType> markerFunctor;
-    typedef vcg::vertex::PointDistanceFunctor<ScalarType> VDistFunct;
+    tri::EmptyTMark<MeshType> markerFunctor;
     std::vector<VertexType*> closests;
     int mergedCnt=0;
     sht.Set(m.vert.begin(), m.vert.end());
@@ -1807,6 +1829,68 @@ public:
     return std::make_pair(TotalCC,DeletedCC);
   }
 
+
+
+  /**
+  Select the folded faces using an angle threshold on the face normal.
+  The face is selected if the dot product between the face normal and the normal of the plane fitted
+  using the vertices of the one ring faces is below the cosThreshold.
+  The cosThreshold requires a negative cosine value (a positive value is clamp to zero).
+  */
+  static void SelectFoldedFaceFromOneRingFaces(MeshType &m, ScalarType cosThreshold)
+  {
+    tri::RequireVFAdjacency(m);
+    tri::RequirePerFaceNormal(m);
+    tri::RequirePerVertexNormal(m);
+    vcg::tri::UpdateSelection<MeshType>::FaceClear(m);
+    vcg::tri::UpdateNormal<MeshType>::PerFaceNormalized(m);
+    vcg::tri::UpdateNormal<MeshType>::PerVertexNormalized(m);
+    vcg::tri::UpdateTopology<MeshType>::VertexFace(m);
+    if (cosThreshold > 0)
+      cosThreshold = 0;
+
+#pragma omp parallel for schedule(dynamic, 10)
+    for (int i = 0; i < m.face.size(); i++)
+    {
+      std::vector<typename MeshType::VertexPointer> nearVertex;
+      std::vector<typename MeshType::CoordType> point;
+      typename MeshType::FacePointer f = &m.face[i];
+      for (int j = 0; j < 3; j++)
+      {
+        std::vector<typename MeshType::VertexPointer> temp;
+        vcg::face::VVStarVF<typename MeshType::FaceType>(f->V(j), temp);
+              typename std::vector<typename MeshType::VertexPointer>::iterator iter = temp.begin();
+        for (; iter != temp.end(); iter++)
+        {
+          if ((*iter) != f->V1(j) && (*iter) != f->V2(j))
+          {
+            nearVertex.push_back((*iter));
+            point.push_back((*iter)->P());
+          }
+        }
+        nearVertex.push_back(f->V(j));
+        point.push_back(f->P(j));
+      }
+
+      if (point.size() > 3)
+      {
+        vcg::Plane3<typename MeshType::ScalarType> plane;
+        vcg::FitPlaneToPointSet(point, plane);
+        float avgDot = 0;
+        for (int j = 0; j < nearVertex.size(); j++)
+          avgDot += plane.Direction().dot(nearVertex[j]->N());
+        avgDot /= nearVertex.size();
+        typename MeshType::VertexType::NormalType normal;
+        if (avgDot < 0)
+          normal = -plane.Direction();
+        else
+          normal = plane.Direction();
+        if (normal.dot(f->N()) < cosThreshold)
+          f->SetS();
+      }
+    }
+  }
+
 }; // end class
 /*@}*/
 
