@@ -29,8 +29,11 @@
 #include <vcg/space/point3.h>
 #include <vcg/space/box3.h>
 #include <vcg/space/point4.h>
+#include <vcg/space/index/grid_util.h>
+#include <vcg/space/index/grid_closest.h>
 #include <vcg/math/base.h>
 #include <vcg/simplex/face/distance.h>
+#include <vcg/simplex/edge/distance.h>
 #include <vcg/simplex/vertex/distance.h>
 #include <vcg/space/intersection3.h>
 #include <vcg/space/index/space_iterators.h>
@@ -64,17 +67,29 @@ namespace vcg {
             FaceTmark(){}
             FaceTmark(MESH_TYPE *m) {this->SetMesh(m);}
         };
+        
+        template <class MESH_TYPE>
+        class EdgeTmark:public Tmark<MESH_TYPE,typename MESH_TYPE::EdgeType>
+        {
+        public:
+            EdgeTmark(){}
+            EdgeTmark(MESH_TYPE *m) {this->SetMesh(m);}
+        };
+        
 
         template <class MESH_TYPE>
-        class VertTmark
+        class EmptyTMark
         {
         public:
         typedef typename  MESH_TYPE::VertexType VertexType;
-            inline VertTmark(){}
-            inline VertTmark(MESH_TYPE *){}
+          typedef typename  MESH_TYPE::EdgeType EdgeType;
+            inline EmptyTMark(){}
+            inline EmptyTMark(MESH_TYPE *){}
             inline void UnMarkAll() const {}
             inline bool IsMarked(VertexType*) const { return false; }
             inline void Mark(VertexType*) const {}
+            inline bool IsMarked(EdgeType*) const { return false; }
+            inline void Mark(EdgeType*) const {}
             inline void SetMesh(void * /*m=0*/) const {}
         };
 
@@ -100,7 +115,6 @@ namespace vcg {
                                                         typename GRID::CoordType & _ip)
         {
             typedef typename GRID::ScalarType ScalarType;
-            typedef Point3<ScalarType> Point3x;
 
             typedef FaceTmark<MESH> MarkerFace;
             MarkerFace mf(&mesh);
@@ -123,39 +137,38 @@ namespace vcg {
             return (0);
         }
 
-		template <class MESH, class GRID>
-		typename MESH::FaceType * GetClosestFaceBase( MESH & mesh,GRID & gr,const typename GRID::CoordType & _p,
-		                                              const typename GRID::ScalarType _maxDist,typename GRID::ScalarType & _minDist,
-		                                              typename GRID::CoordType &_closestPt)
-		{
-			typedef typename GRID::ScalarType ScalarType;
-			typedef Point3<ScalarType> Point3x;
-			typedef FaceTmark<MESH> MarkerFace;
-			MarkerFace mf;
-			mf.SetMesh(&mesh);
-			vcg::face::PointDistanceBaseFunctor<ScalarType> PDistFunct;
-			_minDist=_maxDist;
-			return (gr.GetClosest(PDistFunct,mf,_p,_maxDist,_minDist,_closestPt));
-		}
+    template <class MESH, class GRID>
+    typename MESH::FaceType * GetClosestFaceBase( MESH & mesh,GRID & gr,const typename GRID::CoordType & _p,
+                                                  const typename GRID::ScalarType _maxDist,typename GRID::ScalarType & _minDist,
+                                                  typename GRID::CoordType &_closestPt)
+    {
+      typedef typename GRID::ScalarType ScalarType;
+      typedef FaceTmark<MESH> MarkerFace;
+      MarkerFace mf;
+      mf.SetMesh(&mesh);
+      vcg::face::PointDistanceBaseFunctor<ScalarType> PDistFunct;
+      _minDist=_maxDist;
+      return (gr.GetClosest(PDistFunct,mf,_p,_maxDist,_minDist,_closestPt));
+    }
 
-		template <class MESH, class GRID>
-		typename MESH::FaceType * GetClosestFaceBase( MESH & mesh,GRID & gr,const typename GRID::CoordType & _p,
-		                                              const typename GRID::ScalarType _maxDist,typename GRID::ScalarType & _minDist,
-		                                              typename GRID::CoordType &_closestPt, typename GRID::CoordType & _normf,
-		                                              typename GRID::CoordType & _ip)
-		{
-			typedef typename GRID::ScalarType ScalarType;
-			typename MESH::FaceType * f = GetClosestFaceBase(mesh, gr, _p, _maxDist, _minDist, _closestPt);
-			if(_maxDist> ScalarType(fabs(_minDist)))
-			{
-				// normal computed with trilinear interpolation
-				InterpolationParameters<typename MESH::FaceType,typename MESH::ScalarType>(*f,f->N(),_closestPt, _ip);
-				_normf =  (f->V(0)->cN())*_ip[0]+
-				          (f->V(1)->cN())*_ip[1]+
-				          (f->V(2)->cN())*_ip[2];
-			}
-			return f;
-		}
+    template <class MESH, class GRID>
+    typename MESH::FaceType * GetClosestFaceBase( MESH & mesh,GRID & gr,const typename GRID::CoordType & _p,
+                                                  const typename GRID::ScalarType _maxDist,typename GRID::ScalarType & _minDist,
+                                                  typename GRID::CoordType &_closestPt, typename GRID::CoordType & _normf,
+                                                  typename GRID::CoordType & _ip)
+    {
+      typedef typename GRID::ScalarType ScalarType;
+      typename MESH::FaceType * f = GetClosestFaceBase(mesh, gr, _p, _maxDist, _minDist, _closestPt);
+      if(_maxDist> ScalarType(fabs(_minDist)))
+      {
+        // normal computed with trilinear interpolation
+        InterpolationParameters<typename MESH::FaceType,typename MESH::ScalarType>(*f,f->N(),_closestPt, _ip);
+        _normf =  (f->V(0)->cN())*_ip[0]+
+                  (f->V(1)->cN())*_ip[1]+
+                  (f->V(2)->cN())*_ip[2];
+      }
+      return f;
+    }
 
         template <class MESH, class GRID>
             typename MESH::FaceType * GetClosestFaceEP( MESH & mesh,GRID & gr,const typename GRID::CoordType & _p,
@@ -163,7 +176,6 @@ namespace vcg {
             typename GRID::CoordType &_closestPt)
         {
             typedef typename GRID::ScalarType ScalarType;
-            typedef Point3<ScalarType> Point3x;
             typedef FaceTmark<MESH> MarkerFace;
             MarkerFace mf;
             mf.SetMesh(&mesh);
@@ -177,7 +189,6 @@ namespace vcg {
             const typename GRID::ScalarType & _maxDist,typename GRID::ScalarType & _minDist,
             typename GRID::CoordType &_closestPt)
         {
-            typedef typename GRID::ScalarType ScalarType;
             typedef FaceTmark<MESH> MarkerFace;
             MarkerFace mf;
             mf.SetMesh(&mesh);
@@ -194,7 +205,7 @@ namespace vcg {
         {
             typedef typename GRID::ScalarType ScalarType;
             typedef Point3<ScalarType> Point3x;
-            typedef VertTmark<MESH> MarkerVert;
+            typedef EmptyTMark<MESH> MarkerVert;
             MarkerVert mv;
             mv.SetMesh(&mesh);
             typedef vcg::vertex::PointDistanceFunctor<typename MESH::ScalarType> VDistFunct;
@@ -210,7 +221,7 @@ namespace vcg {
         {
             typedef typename GRID::ScalarType ScalarType;
             typedef Point3<ScalarType> Point3x;
-            typedef VertTmark<MESH> MarkerVert;
+            typedef EmptyTMark<MESH> MarkerVert;
             MarkerVert mv;
             mv.SetMesh(&mesh);
             typedef vcg::vertex::PointScaledDistanceFunctor<typename MESH::VertexType> VDistFunct;
@@ -227,7 +238,7 @@ namespace vcg {
         {
             typedef typename GRID::ScalarType ScalarType;
             typedef Point3<ScalarType> Point3x;
-            typedef VertTmark<MESH> MarkerVert;
+            typedef EmptyTMark<MESH> MarkerVert;
             MarkerVert mv;
             mv.SetMesh(&mesh);
             typedef vcg::vertex::PointNormalDistanceFunctor<typename MESH::VertexType> VDistFunct;
@@ -270,7 +281,7 @@ namespace vcg {
             const typename GRID::CoordType & _p, const typename GRID::ScalarType & _maxDist,
             OBJPTRCONTAINER & _objectPtrs,DISTCONTAINER & _distances, POINTCONTAINER & _points)
         {
-            typedef VertTmark<MESH> MarkerVert;
+            typedef EmptyTMark<MESH> MarkerVert;
             MarkerVert mv;
             mv.SetMesh(&mesh);
             typedef vcg::vertex::PointDistanceFunctor<typename MESH::ScalarType> VDistFunct;
@@ -305,7 +316,7 @@ namespace vcg {
             DISTCONTAINER & _distances,
             POINTCONTAINER & _points)
         {
-            typedef VertTmark<MESH> MarkerVert;
+            typedef EmptyTMark<MESH> MarkerVert;
             MarkerVert mv;
             mv.SetMesh(&mesh);
             typedef vcg::vertex::PointDistanceFunctor<typename MESH::ScalarType> VDistFunct;
@@ -331,7 +342,7 @@ namespace vcg {
             const vcg::Box3<typename GRID::ScalarType> _bbox,
             OBJPTRCONTAINER & _objectPtrs)
         {
-            typedef VertTmark<MESH> MarkerVert;
+            typedef EmptyTMark<MESH> MarkerVert;
             MarkerVert mv;
             mv.SetMesh(&mesh);
             return(gr.GetInBox/*<MarkerVert,OBJPTRCONTAINER>*/(mv,_bbox,_objectPtrs));
@@ -341,8 +352,6 @@ namespace vcg {
             typename GRID::ObjPtr DoRay(MESH & mesh,GRID & gr, const Ray3<typename GRID::ScalarType> & _ray,
             const typename GRID::ScalarType & _maxDist, typename GRID::ScalarType & _t)
         {
-            typedef typename MESH::FaceType FaceType;
-            typedef typename MESH::ScalarType ScalarType;
             typedef FaceTmark<MESH> MarkerFace;
             MarkerFace mf;
             mf.SetMesh(&mesh);
@@ -458,14 +467,14 @@ namespace vcg {
         };
 
         template <class GRID,class MESH>
-        class ClosestVertexIterator:public vcg::ClosestIterator<GRID, vcg::vertex::PointDistanceFunctor<typename MESH::ScalarType>, VertTmark<MESH> >
+        class ClosestVertexIterator:public vcg::ClosestIterator<GRID, vcg::vertex::PointDistanceFunctor<typename MESH::ScalarType>, EmptyTMark<MESH> >
         {
         public:
             typedef GRID GridType;
             typedef MESH MeshType;
-            typedef VertTmark<MESH> MarkerVert;
+            typedef EmptyTMark<MESH> MarkerVert;
             typedef vcg::vertex::PointDistanceFunctor<typename MESH::ScalarType> VDistFunct;
-            typedef vcg::ClosestIterator<GRID, VDistFunct, VertTmark<MESH> > ClosestBaseType;
+            typedef vcg::ClosestIterator<GRID, VDistFunct, EmptyTMark<MESH> > ClosestBaseType;
             VDistFunct fn;
             ClosestVertexIterator(GridType &_Si):ClosestBaseType(_Si,fn){}
 
@@ -496,12 +505,11 @@ namespace vcg {
 
         /// this is for meshes with 2D coordinates
         template <class MESH, class GRID>
-            typename MESH::FaceType * GetClosestEdgeBase( MESH & mesh,GRID & gr,const typename GRID::CoordType & _p,
+            typename MESH::EdgeType * GetClosestEdge2DBase( MESH & mesh,GRID & gr,const typename GRID::CoordType & _p,
                                                           const typename GRID::ScalarType _maxDist,typename GRID::ScalarType & _minDist,
                                                           typename GRID::CoordType &_closestPt)
         {
             typedef typename GRID::ScalarType ScalarType;
-            typedef Point3<ScalarType> Point3x;
             typedef FaceTmark<MESH> MarkerFace;
             MarkerFace mf;
             mf.SetMesh(&mesh);
@@ -510,6 +518,21 @@ namespace vcg {
             return (gr.GetClosest(PDistFunct,mf,_p,_maxDist,_minDist,_closestPt));
         }
 
+        /// this is for meshes with 3D coordinates
+        template <class MESH, class GRID>
+            typename MESH::EdgeType * GetClosestEdgeBase( MESH & mesh,GRID & gr,const typename GRID::CoordType & _p,
+                                                          const typename GRID::ScalarType _maxDist,typename GRID::ScalarType & _minDist,
+                                                          typename GRID::CoordType &_closestPt)
+        {
+            typedef typename GRID::ScalarType ScalarType;
+            typedef EmptyTMark<MESH> MarkerEdge;
+            MarkerEdge mf;
+            mf.SetMesh(&mesh);
+            vcg::edge::PointDistanceFunctor<ScalarType> PDistFunct;
+            _minDist=_maxDist;
+//            return (gr.GetClosest(PDistFunct,mf,_p,_maxDist,_minDist,_closestPt));
+            return vcg::GridClosest(gr,PDistFunct,mf, _p,_maxDist,_minDist,_closestPt);
+        }
 
     }	 // end namespace tri
 }	 // end namespace vcg
