@@ -50,7 +50,6 @@ namespace tri {
 template <class TetraMeshType>
 void Tetrahedron(TetraMeshType &in)
 {
- typedef TetraMeshType MeshType;
  typedef typename TetraMeshType::CoordType CoordType;
  typedef typename TetraMeshType::VertexPointer  VertexPointer;
  typedef typename TetraMeshType::VertexIterator VertexIterator;
@@ -278,7 +277,6 @@ void Icosahedron(IcoMeshType &in)
 template <class MeshType>
 void Hexahedron(MeshType &in)
 {
- typedef typename MeshType::ScalarType ScalarType;
  typedef typename MeshType::CoordType CoordType;
  typedef typename MeshType::VertexPointer  VertexPointer;
  typedef typename MeshType::VertexIterator VertexIterator;
@@ -327,7 +325,6 @@ void Hexahedron(MeshType &in)
 template <class MeshType>
 void Square(MeshType &in)
 {
-  typedef typename MeshType::ScalarType ScalarType;
   typedef typename MeshType::CoordType CoordType;
   typedef typename MeshType::VertexPointer  VertexPointer;
   typedef typename MeshType::VertexIterator VertexIterator;
@@ -405,9 +402,7 @@ void SphericalCap(MeshType &in, float angleRad, const int subdiv = 3 )
 template <class MeshType>
 void Sphere(MeshType &in, const int subdiv = 3 )
 {
- typedef typename MeshType::ScalarType ScalarType;
  typedef typename MeshType::CoordType CoordType;
- typedef typename MeshType::VertexPointer  VertexPointer;
  typedef typename MeshType::VertexIterator VertexIterator;
  typedef typename MeshType::FaceIterator   FaceIterator;
     if(in.vn==0 && in.fn==0) Icosahedron(in);
@@ -445,7 +440,6 @@ void Cone( MeshType& in,
           const typename MeshType::ScalarType h,
           const int SubDiv = 36  )
 {
- typedef typename MeshType::ScalarType ScalarType;
  typedef typename MeshType::CoordType CoordType;
  typedef typename MeshType::VertexPointer  VertexPointer;
  typedef typename MeshType::VertexIterator VertexIterator;
@@ -534,7 +528,6 @@ void Cone( MeshType& in,
 template <class MeshType >
 void Box(MeshType &in, const typename MeshType::BoxType & bb )
 {
- typedef typename MeshType::ScalarType ScalarType;
  typedef typename MeshType::CoordType CoordType;
  typedef typename MeshType::VertexPointer  VertexPointer;
  typedef typename MeshType::VertexIterator VertexIterator;
@@ -590,34 +583,120 @@ void Torus(MeshType &m, float hRingRadius, float vRingRadius, int hRingDiv=24, i
   Allocator<MeshType>::AddVertices(m,(vRingDiv+1)*(hRingDiv+1));
   for(int i=0;i<hRingDiv+1;++i)
   {
-    Matrix44x RotM; RotM.SetRotateRad(float(i%hRingDiv)*angleStepH,CoordType(0,1,0));
+    Matrix44x RotM; RotM.SetRotateRad(float(i%hRingDiv)*angleStepH,CoordType(0,0,1));
     for(int j=0;j<vRingDiv+1;++j)
     {
       CoordType p;
       p[0]= vRingRadius*cos(float(j%vRingDiv)*angleStepV) + hRingRadius;
-      p[1]= vRingRadius*sin(float(j%vRingDiv)*angleStepV);
-      p[2] = 0;
+      p[1] = 0;
+      p[2]= vRingRadius*sin(float(j%vRingDiv)*angleStepV);
 
       m.vert[i*(vRingDiv+1)+j].P() = RotM*p;
     }
   }
   FaceGrid(m,vRingDiv+1,hRingDiv+1);
   tri::Clean<MeshType>::RemoveDuplicateVertex(m);
-  tri::Allocator<MeshType>::CompactVertexVector(m);
+  tri::Allocator<MeshType>::CompactEveryVector(m);
 
 }
 
+/// Auxilary functions for superquadric surfaces
+/// Used by SuperToroid and SuperEllipsoid
+template <class ScalarType>
+static  ScalarType _SQfnC(ScalarType a, ScalarType b){
+  return math::Sgn(cos(a))*pow(fabs(cos(a)),b);
+};
+template <class ScalarType>
+static ScalarType _SQfnS(ScalarType a, ScalarType b){
+  return math::Sgn(sin(a))*pow(fabs(sin(a)),b);
+};
 
+
+/**
+ * SuperToroid
+ * 
+ * Generate a  a supertoroid, e.g. a member of a family of doughnut-like surfaces 
+ * (technically, a topological torus) whose shape is defined by mathematical formulas 
+ * similar to those that define the superquadrics. 
+ */
+template <class MeshType>
+void SuperToroid(MeshType &m, float hRingRadius, float vRingRadius, float vSquareness, float hSquareness, int hRingDiv=24, int vRingDiv=12 )
+{
+  typedef typename MeshType::CoordType CoordType;
+  typedef typename MeshType::ScalarType ScalarType;
+  m.Clear();
+  ScalarType angleStepV = (2.0f*M_PI)/vRingDiv;
+  ScalarType angleStepH = (2.0f*M_PI)/hRingDiv;
+  
+  ScalarType u,v;
+  int count;
+  Allocator<MeshType>::AddVertices(m,(vRingDiv+1)*(hRingDiv+1));
+  for(int i=0;i<hRingDiv+1;++i)
+  {
+    u=float(i%hRingDiv)*angleStepH;
+    count=0;
+    for(int j=vRingDiv;j>=0;--j)
+    {
+      CoordType p;
+      v=float(j%vRingDiv)*angleStepV;
+      p[0]= (hRingRadius+vRingRadius*_SQfnC(u,vSquareness))*_SQfnC(v,hSquareness);;
+      p[1]= (hRingRadius+vRingRadius*_SQfnC(u,vSquareness))*_SQfnS(v,hSquareness);
+      p[2] = vRingRadius*_SQfnS(u,vSquareness);
+      m.vert[i*(vRingDiv+1)+count].P() = p;
+      count++;
+    }
+  }
+  FaceGrid(m,vRingDiv+1,hRingDiv+1);
+  tri::Clean<MeshType>::RemoveDuplicateVertex(m);
+  tri::Allocator<MeshType>::CompactEveryVector(m);
+
+}
+/**
+ * Generate a SuperEllipsoid eg  a solid whose horizontal sections are super-ellipses (Lamé curves)
+ * with the same exponent r, and whose vertical sections through the center are super-ellipses with 
+ * the same exponent t.
+ */
+template <class MeshType>
+void SuperEllipsoid(MeshType &m, float rFeature, float sFeature, float tFeature, int hRingDiv=24, int vRingDiv=12 )
+{
+  typedef typename MeshType::CoordType CoordType;
+  typedef typename MeshType::ScalarType ScalarType;
+  m.Clear();
+  ScalarType angleStepV = (2.0f*M_PI)/vRingDiv;
+  ScalarType angleStepH = (1.0f*M_PI)/hRingDiv;
+  float u;
+  float v;
+  Allocator<MeshType>::AddVertices(m,(vRingDiv+1)*(hRingDiv+1));
+  for(int i=0;i<hRingDiv+1;++i)
+  {
+    //u=ScalarType(i%hRingDiv)*angleStepH + angleStepH/2.0;
+    u=i*angleStepH;
+    for(int j=0;j<vRingDiv+1;++j)
+    {
+      CoordType p;
+      v=ScalarType(j%vRingDiv)*angleStepV;
+      p[0] = _SQfnC(v,2/rFeature)*_SQfnC(u,2/rFeature);
+      p[1] = _SQfnC(v,2/sFeature)*_SQfnS(u,2/sFeature);
+      p[2] = _SQfnS(v,2/tFeature);
+      m.vert[i*(vRingDiv+1)+j].P() = p;
+    }
+  }
+  FaceGrid(m,vRingDiv+1,hRingDiv+1);
+  tri::Clean<MeshType>::MergeCloseVertex(m,ScalarType(angleStepV*angleStepV*0.001));
+  tri::Allocator<MeshType>::CompactEveryVector(m);
+  bool oriented, orientable;
+  tri::UpdateTopology<MeshType>::FaceFace(m);
+  tri::Clean<MeshType>::OrientCoherentlyMesh(m,oriented,orientable);  
+  tri::UpdateSelection<MeshType>::Clear(m);
+}
 // this function build a mesh starting from a vector of generic coords (objects having a triple of float at their beginning)
 // and a vector of faces (objects having a triple of ints at theri beginning).
 template <class MeshType,class V, class F >
-void Build( MeshType & in, const V & v, const F & f)
+void BuildMeshFromCoordVectorIndexVector( MeshType & in, const V & v, const F & f)
 {
-  typedef typename MeshType::ScalarType ScalarType;
   typedef typename MeshType::CoordType CoordType;
   typedef typename MeshType::VertexPointer  VertexPointer;
   typedef typename MeshType::VertexIterator VertexIterator;
-  typedef typename MeshType::FaceIterator   FaceIterator;
 
   in.Clear();
   Allocator<MeshType>::AddVertices(in,v.size());
@@ -654,10 +733,10 @@ void Build( MeshType & in, const V & v, const F & f)
 
 
 template <class MeshType,class V>
-void Build( MeshType & in, const V & v)
+void BuildMeshFromCoordVector( MeshType & in, const V & v)
 {
   std::vector<Point3i> dummyfaceVec;
-  Build(in,v,dummyfaceVec);
+  BuildMeshFromCoordVectorIndexVector(in,v,dummyfaceVec);
 }
 
 
@@ -698,9 +777,6 @@ template <class MeshType>
 void Grid(MeshType & in, int w, int h, float wl, float hl, float *data=0)
 {
   typedef typename MeshType::CoordType CoordType;
-  typedef typename MeshType::VertexPointer  VertexPointer;
-  typedef typename MeshType::VertexIterator VertexIterator;
-  typedef typename MeshType::FaceIterator   FaceIterator;
 
   in.Clear();
   Allocator<MeshType>::AddVertices(in,w*h);
@@ -764,7 +840,7 @@ void FaceGrid(MeshType & in, int w, int h)
 template <class MeshType>
 void FaceGrid(MeshType & in, const std::vector<int> &grid, int w, int h)
 {
-    assert(in.vn == (int)in.vert.size()); // require a compact vertex vector
+    tri::RequireCompactness(in);
     assert(in.vn <= w*h); // the number of vertices should match the number of expected grid vertices
 
 //	    V0       V1
