@@ -809,6 +809,55 @@ public:
         return alpha;
     }
 
+    ///transform a cross field into a couple of angles
+    static void CrossFieldToAngles(const FaceType &f,
+                                   ScalarType &alpha1,
+                                   ScalarType &alpha2,
+                                   int RefEdge=1)
+    {
+        CoordType axis0=f.cP1(RefEdge)-f.cP0(RefEdge);
+        axis0.Normalize();
+        CoordType axis2=f.cN();
+        axis2.Normalize();
+        CoordType axis1=axis2^axis0;
+        axis1.Normalize();
+
+
+        vcg::Matrix33<ScalarType> Trans=vcg::TransformationMatrix(axis0,axis1,axis2);
+
+        //trensform the vector to the reference frame by rotating it
+        CoordType trasfPD1=Trans*f.cPD1();
+        CoordType trasfPD2=Trans*f.cPD2();
+
+        //then find the angle with respact to axis 0
+        alpha1=atan2(trasfPD1.Y(),trasfPD1.X());
+        alpha2=atan2(trasfPD2.Y(),trasfPD2.X());
+    }
+
+    ///transform a cross field into a couple of angles
+    static void AnglesToCrossField(FaceType &f,
+                                   const ScalarType &alpha1,
+                                   const ScalarType &alpha2,
+                                   int RefEdge=1)
+    {
+          CoordType axis0=f.cP1(RefEdge)-f.cP0(RefEdge);
+          axis0.Normalize();
+          CoordType axis2=f.cN();
+          axis2.Normalize();
+          CoordType axis1=axis2^axis0;
+          axis1.Normalize();
+
+          vcg::Matrix33<ScalarType> Trans=vcg::TransformationMatrix(axis0,axis1,axis2);
+          vcg::Matrix33<ScalarType> InvTrans=Inverse(Trans);
+
+          CoordType PD1=CoordType(cos(alpha1),sin(alpha1),0);
+          CoordType PD2=CoordType(cos(alpha2),sin(alpha2),0);
+
+          //then transform and store in the face
+          f.PD1()=(InvTrans*PD1);
+          f.PD2()=(InvTrans*PD2);
+    }
+
     ///return the 4 directiona of the cross field in 3D
     ///given a first direction as input
     static void CrossVector(const CoordType &dir0,
@@ -1140,34 +1189,6 @@ public:
     }
 
 
-//    ///return true if a given vertex is singular,
-//    ///return also the missmatch
-//    static bool IsSingularByCross(const VertexType &v,int &missmatch)
-//    {
-//        typedef typename VertexType::FaceType FaceType;
-//        ///check that is on border..
-//        if (v.IsB())return false;
-
-//        std::vector<face::Pos<FaceType> > posVec;
-//        //SortedFaces(v,faces);
-//        face::Pos<FaceType> pos(v.cVFp(), v.cVFi());
-//        vcg::face::VFOrderedStarFF(pos, posVec);
-
-//        missmatch=0;
-//        for (unsigned int i=0;i<posVec.size();i++)
-//        {
-//            FaceType *curr_f=posVec[i].F();
-//            FaceType *next_f=posVec[(i+1)%posVec.size()].F();
-
-//            ///find the current missmatch
-//            missmatch+=MissMatchByCross(*curr_f,*next_f);
-
-//            missmatch=missmatch%4;
-//        }
-////        missmatch=missmatch%4;
-//        return(missmatch!=0);
-//    }
-
     ///return true if a given vertex is singular,
     ///return also the missmatch
     static bool IsSingularByCross(const VertexType &v,int &missmatch)
@@ -1188,9 +1209,7 @@ public:
             FaceType *next_f=posVec[(i+1)%posVec.size()].F();
 
             //find the current missmatch
-            //missmatch+=MissMatchByCross(*curr_f,*next_f);
             curr_dir=FollowDirection(*curr_f,*next_f,curr_dir);
-            //missmatch=missmatch%4;
         }
         missmatch=curr_dir;
         return(curr_dir!=0);
@@ -1242,20 +1261,64 @@ public:
                                 CoordType &dirU,
                                 CoordType &dirV)
     {
-        ///compute non normalized normal
-        CoordType n  =  f.cN();
+        vcg::Point2<ScalarType> Origin2D=(UV0+UV1+UV2)/3;
+        CoordType Origin3D=(f.cP(0)+f.cP(1)+f.cP(2))/3;
 
-        CoordType p0 =f.cP(1) - f.cP(0);
-        CoordType p1 =f.cP(2) - f.cP(1);
-        CoordType p2 =f.cP(0) - f.cP(2);
+        vcg::Point2<ScalarType> UvT0=UV0-Origin2D;
+        vcg::Point2<ScalarType> UvT1=UV1-Origin2D;
+        vcg::Point2<ScalarType> UvT2=UV2-Origin2D;
 
-        CoordType t[3];
-        t[0] =  -(p0 ^ n);
-        t[1] =  -(p1 ^ n);
-        t[2] =  -(p2 ^ n);
+        CoordType PosT0=f.cP(0)-Origin3D;
+        CoordType PosT1=f.cP(1)-Origin3D;
+        CoordType PosT2=f.cP(2)-Origin3D;
 
-        dirU = t[1]*UV0.X() + t[2]*UV1.X() + t[0]*UV2.X();
-        dirV = t[1]*UV0.Y() + t[2]*UV1.Y() + t[0]*UV2.Y();
+        CoordType Bary0,Bary1;
+        vcg::InterpolationParameters2(UvT0,UvT1,UvT2,vcg::Point2<ScalarType>(1,0),Bary0);
+        vcg::InterpolationParameters2(UvT0,UvT1,UvT2,vcg::Point2<ScalarType>(0,1),Bary1);
+
+        //then transport to 3D
+        dirU=PosT0*Bary0.X()+PosT1*Bary0.Y()+PosT2*Bary0.Z();
+        dirV=PosT0*Bary1.X()+PosT1*Bary1.Y()+PosT2*Bary1.Z();
+
+//        dirU-=Origin3D;
+//        dirV-=Origin3D;
+        dirU.Normalize();
+        dirV.Normalize();
+        //orient coherently
+        CoordType Ntest=dirU^dirV;
+        CoordType NTarget=vcg::Normal(f.cP(0),f.cP(1),f.cP(2));
+        if ((Ntest*NTarget)<0)dirV=-dirV;
+
+//        //then make them orthogonal
+//        CoordType dirAvg=dirU^dirV;
+        CoordType dirVTarget=NTarget^dirU;
+        CoordType dirUTarget=NTarget^dirV;
+
+         dirUTarget.Normalize();
+         dirVTarget.Normalize();
+         if ((dirUTarget*dirU)<0)dirUTarget=-dirUTarget;
+         if ((dirVTarget*dirV)<0)dirVTarget=-dirVTarget;
+
+         dirU=(dirU+dirUTarget)/2;
+         dirV=(dirV+dirVTarget)/2;
+
+         dirU.Normalize();
+         dirV.Normalize();
+
+//        ///compute non normalized normal
+//        CoordType n  =  f.cN();
+
+//        CoordType p0 =f.cP(1) - f.cP(0);
+//        CoordType p1 =f.cP(2) - f.cP(1);
+//        CoordType p2 =f.cP(0) - f.cP(2);
+
+//        CoordType t[3];
+//        t[0] =  -(p0 ^ n);
+//        t[1] =  -(p1 ^ n);
+//        t[2] =  -(p2 ^ n);
+
+//        dirU = t[1]*UV0.X() + t[2]*UV1.X() + t[0]*UV2.X();
+//        dirV = t[1]*UV0.Y() + t[2]*UV1.Y() + t[0]*UV2.Y();
     }
 
     static void MakeDirectionFaceCoherent(FaceType *f0,
@@ -1298,7 +1361,7 @@ public:
         {
             FaceType *f=&mesh.face[i];
             if (f->IsD())continue;
-            CoordType Ntest=mesh.face[i].PD1()^mesh.face[i].PD2();
+            CoordType Ntest= CoordType::Construct( mesh.face[i].PD1()^mesh.face[i].PD2() );
             if ((Ntest*vcg::Normal(f->P(0),f->P(1),f->P(2)))<0)mesh.face[i].PD2()=-mesh.face[i].PD2();
         }
     }
@@ -1401,6 +1464,20 @@ public:
         return curvUV;
     }
 
+    static void InitDirFromWEdgeUV(MeshType &mesh)
+    {
+        for (size_t i=0;i<mesh.face.size();i++)
+        {
+            vcg::Point2<ScalarType> UV0=mesh.face[i].WT(0).P();
+            vcg::Point2<ScalarType> UV1=mesh.face[i].WT(1).P();
+            vcg::Point2<ScalarType> UV2=mesh.face[i].WT(2).P();
+            GradientToCross(mesh.face[i],UV0,UV1,UV2,
+                            CoordType::Construct(mesh.face[i].PD1()),
+                            CoordType::Construct(mesh.face[i].PD2()) );
+        }
+        OrientDirectionFaceCoherently(mesh);
+    }
+
 };///end class
 } //End Namespace Tri
 } // End Namespace vcg
