@@ -68,14 +68,17 @@ public:
   * @brief The PC_ResultCode enum codifies the result type of a polychord collapse operation.
   */
   enum PC_ResultCode {
-    PC_SUCCESS = 0,
-    PC_NOTMANIF = 1,
-    PC_NOTQUAD = 2,
-    PC_NOLINKCOND = 4,
-    PC_SINGBOTH = 8,
-    PC_SELFINTERSECT = 16,
-    PC_VOID = 32,
-    PC_OTHER = 64
+    PC_SUCCESS          = 0x000,
+    PC_NOTMANIF         = 0x001,
+    PC_NOTQUAD          = 0x002,
+    PC_NOLINKCOND       = 0x004,
+    PC_SINGSIDEA        = 0x008,
+    PC_SINGSIDEB        = 0x010,
+    PC_SINGBOTH         = 0x020,
+    PC_SELFINTERSECT    = 0x040,
+    PC_NOMOREMANIF      = 0x080,
+    PC_VOID             = 0x100,
+    PC_OTHER            = 0x100
   };
 
   /**
@@ -112,9 +115,8 @@ public:
      * @brief ResetMarks
      */
     void ResetMarks() {
-      typename std::vector<PC_Chord>::iterator it = _Chords.begin();
-      for (; it != _Chords.end(); it++)
-        (*it).mark = std::numeric_limits<unsigned long>::max();
+      for (size_t i = 0; i < _Chords.size(); ++i)
+        _Chords.at(i).mark = std::numeric_limits<unsigned long>::max();
     }
 
     /**
@@ -124,13 +126,13 @@ public:
      */
     void Reset(const PolyMeshType &mesh) {
       _Chords.resize(2*mesh.face.size());
-      for (size_t j = 0; j < _Chords.size(); j++)
+      for (size_t j = 0; j < _Chords.size(); ++j)
         _Chords[j].Reset();
       _currentChord = NULL;
 
       PC_Chord *chord = NULL;
       long long j = 0;
-      for (size_t i = 0; i < _Chords.size(); i++) {
+      for (size_t i = 0; i < _Chords.size(); ++i) {
         // set the prev
         chord = NULL;
         if ((long long)i-1 >= 0) {
@@ -138,7 +140,7 @@ public:
           if (vcg::tri::HasPerFaceFlags(mesh)) {
             j = i-1;
             while (j >= 0 && mesh.face[j/2].IsD())
-              j--;
+              --j;
             if (j >= 0)
               chord = &_Chords[j];
             else
@@ -154,7 +156,7 @@ public:
           if (vcg::tri::HasPerFaceFlags(mesh)) {
             j = i+1;
             while (j < (long long)_Chords.size() && mesh.face[j/2].IsD())
-              j++;
+              ++j;
             if (j < (long long)_Chords.size())
               chord = &_Chords[j];
             else
@@ -213,8 +215,8 @@ public:
           coord.prev->next = coord.next;
         if (coord.next != NULL && &coord != _currentChord)
           coord.next->prev = coord.prev;
-        coord.mark = mark;
       }
+      coord.mark = mark;
       coord.q = resultCode;
     }
 
@@ -372,7 +374,7 @@ public:
      * @brief LC_ResetStars resets the stars on a polychord.
      */
     void LC_ResetStars() {
-      for (size_t v = 0; v < _lcVertices.size(); v++)
+      for (size_t v = 0; v < _lcVertices.size(); ++v)
         _lcVertices[v].reset();
     }
 
@@ -394,7 +396,7 @@ public:
 
       // count how many edges
       do {
-        nEdges++;
+        ++nEdges;
         // go on the next edge
         runPos.FlipE();
         runPos.FlipV();
@@ -402,11 +404,11 @@ public:
         runPos.FlipF();
       } while (runPos != startPos && !runPos.IsBorder());
       if (runPos.IsBorder())
-        nEdges++;
+        ++nEdges;
 
       // resize the vector of edges
       lcEdges.resize(nEdges);
-      for (size_t e = 0; e < nEdges; e++)
+      for (size_t e = 0; e < nEdges; ++e)
         lcEdges[e].reset();
 
       /// compute the star of all the vertices and edges seen from the polychord
@@ -524,7 +526,7 @@ public:
       _lcVertices[v2].star.erase(v1);     // remove v1 from v2-star
       // foreach v | v2 \in star(v) [i.e. v \in star(v2)]
       //      star(v) = star(v) U {v1} \ {v2}
-      for (typename LCVertexStar::iterator vIt = _lcVertices[v2].star.begin(); vIt != _lcVertices[v2].star.end(); vIt++) {
+      for (typename LCVertexStar::iterator vIt = _lcVertices[v2].star.begin(); vIt != _lcVertices[v2].star.end(); ++vIt) {
         v = *vIt;
         if (v == v2)  // skip v2 itself
           continue;
@@ -534,9 +536,9 @@ public:
       /// update the star of the edges which include v1 and v2 in their star
       // foreach e | v1 \in star(e) ^ v2 \in star(e)
       //      star(e) = star(e) \ {v1,v2} U {v1}
-      for (typename LCEdgeStar::iterator eIt = _lcVertices[v1].edges.begin(); eIt != _lcVertices[v1].edges.end(); eIt++)
+      for (typename LCEdgeStar::iterator eIt = _lcVertices[v1].edges.begin(); eIt != _lcVertices[v1].edges.end(); ++eIt)
         lcEdges[*eIt].star.erase(v2);
-      for (typename LCEdgeStar::iterator eIt = _lcVertices[v2].edges.begin(); eIt != _lcVertices[v2].edges.end(); eIt++) {
+      for (typename LCEdgeStar::iterator eIt = _lcVertices[v2].edges.begin(); eIt != _lcVertices[v2].edges.end(); ++eIt) {
         lcEdges[*eIt].star.erase(v2);
         lcEdges[*eIt].star.insert(v1);
       }
@@ -551,6 +553,58 @@ public:
 
   // PolychordCollapse's methods begin here::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
+//  /**
+//   * @brief CheckConsistent checks for consistency. ONLY FOR DEBUG.
+//   * @param mesh
+//   * @return
+//   */
+//  static bool CheckConsistent(PolyMeshType &mesh) {
+//    vcg::tri::RequirePerFaceFlags(mesh);
+//    vcg::tri::RequirePerFaceColor(mesh);
+//    for (size_t f = 0; f < mesh.face.size(); ++f) {
+//      if (!mesh.face[f].IsD()) {
+//        for (int v = 0; v < mesh.face[f].VN(); ++v) {
+//          if (!vcg::face::IsBorder(mesh.face[f], v)) {
+//            if (mesh.face[f].FFp(v)->IsD()) {
+//              mesh.face[f].C() = vcg::Color4b(vcg::Color4b::Magenta);
+//              return false;
+//            }
+//            if (mesh.face[f].FFp(v)->FFp(mesh.face[f].FFi(v)) != &mesh.face[f]) {
+//              mesh.face[f].C() = vcg::Color4b(vcg::Color4b::Yellow);
+//              return false;
+//            }
+//          }
+//        }
+//      }
+//    }
+//    return true;
+//  }
+
+  /**
+   * @brief MarkPolychords marks the chords of the polychord starting at startPos.
+   * @param mesh The input mesh.
+   * @param startPos The starting position.
+   * @param chords The vector of chords.
+   * @param mark The current mark, used to identify quads already visited.
+   */
+  static void MarkPolychords(const PolyMeshType &mesh,
+                             const vcg::face::Pos<FaceType> &startPos,
+                             PC_Chords &chords,
+                             const unsigned long mark) {
+    vcg::face::Pos<FaceType> runPos = startPos;
+    std::pair<size_t, unsigned char> face_edge(std::numeric_limits<size_t>::max(), 0);
+    do {
+      assert(runPos.F()->VN() == 4);
+      face_edge.first = vcg::tri::Index(mesh, runPos.F());
+      face_edge.second = runPos.E() % 2;
+      chords[face_edge].mark = mark;
+      runPos.FlipE();
+      runPos.FlipV();
+      runPos.FlipE();
+      runPos.FlipF();
+    } while (runPos != startPos && !runPos.IsBorder());
+  }
+
   /**
    * @brief CollapsePolychord performs all checks and then collapses the polychord.
    *
@@ -600,7 +654,7 @@ public:
     // check if the sequence of facets is a polychord and find the starting coord
     PC_ResultCode resultCode = CheckPolychordFindStartPosition(pos, startPos, checkSing);
     // if not successful, visit the sequence for marking it and return
-    if (resultCode != PC_SUCCESS) {
+    if (resultCode != PC_SUCCESS && resultCode != PC_SINGSIDEA && resultCode != PC_SINGSIDEB) {
       // if not manifold, visit the entire polychord ending on the non-manifold edge
       if (resultCode == PC_NOTMANIF) {
         tempPos = pos;
@@ -623,33 +677,41 @@ public:
           tempPos.FlipV();
           tempPos.FlipE();
         } while (tempPos != startPos);
+        VisitPolychord(mesh, startPos, chords, mark, resultCode);
+        return resultCode;
       }
       VisitPolychord(mesh, startPos, chords, mark, resultCode);
       return resultCode;
     }
     // check if the link conditions are satisfied
-    bool lc = linkConditions.CheckLinkConditions(mesh, startPos);
     // if not satisfied, visit the sequence for marking it and return
-    if (!lc) {
+    if (!linkConditions.CheckLinkConditions(mesh, startPos)) {
       VisitPolychord(mesh, startPos, chords, mark, PC_NOLINKCOND);
       return PC_NOLINKCOND;
     }
+    // mark the polychord's chords
+    MarkPolychords(mesh, startPos, chords, mark);
     // check if the polychord does not intersect itself
-    bool si = IsPolychordSelfIntersecting(mesh, startPos, chords, mark);
     // if it self-intersects, visit the polychord for marking it and return
-    if (si) {
+    if (IsPolychordSelfIntersecting(mesh, startPos, chords, mark)) {
       VisitPolychord(mesh, startPos, chords, mark, PC_SELFINTERSECT);
       return PC_SELFINTERSECT;
     }
+    // check if manifoldness remains
+    // if it will loose manifoldness, visit the sequence for marking it and return
+    if (!WillPolychordBeManifold(mesh, startPos, chords, mark)) {
+      VisitPolychord(mesh, startPos, chords, mark, PC_NOMOREMANIF);
+      return PC_NOMOREMANIF;
+    }
     // at this point the polychord is collapsable, visit it for marking
     VisitPolychord(mesh, startPos, chords, mark, PC_SUCCESS);
 
     // now collapse
     CoordType point;
-    int valenceA = 0, valenceB = 0;
+//    int valenceA = 0, valenceB = 0;
     vcg::face::Pos<FaceType> runPos = startPos;
     vcg::face::JumpingPos<FaceType> tmpPos;
-    bool onSideA = false, onSideB = false;
+//    bool onSideA = false, onSideB = false;
     vcg::face::Pos<FaceType> sideA, sideB;
     typedef std::queue<VertexPointer *> FacesVertex;
     typedef std::pair<VertexPointer, FacesVertex> FacesVertexPair;
@@ -663,43 +725,14 @@ public:
     std::queue<VertexPointer> verticesToDeleteQueue;
     std::queue<FacePointer> facesToDeleteQueue;
 
-    if (checkSing) {
-      do {
-        runPos.FlipV();
-        valenceB = runPos.NumberOfIncidentVertices();
-        tmpPos.Set(runPos.F(), runPos.E(), runPos.V());
-        if (tmpPos.FindBorder())
-          valenceB++;
-        runPos.FlipV();
-        valenceA = runPos.NumberOfIncidentVertices();
-        tmpPos.Set(runPos.F(), runPos.E(), runPos.V());
-        if (tmpPos.FindBorder())
-          valenceA++;
-        if (valenceA != 4)
-          onSideA = true;
-        if (valenceB != 4)
-          onSideB = true;
-        assert(!onSideA || !onSideB);
-
-        if (runPos != startPos && runPos.IsBorder())
-          break;
-
-        // go on next edge/face
-        runPos.FlipE();
-        runPos.FlipV();
-        runPos.FlipE();
-        runPos.FlipF();
-      } while (runPos != startPos);
-    }
-
     runPos = startPos;
     do {
       // compute new vertex
       point = (runPos.V()->P() + runPos.VFlip()->P()) / 2.f;
       if (checkSing) {
-        if (onSideA)
+        if (resultCode == PC_SINGSIDEA)
           point = runPos.V()->P();
-        if (onSideB)
+        else if (resultCode == PC_SINGSIDEB)
           point = runPos.VFlip()->P();
       }
       runPos.V()->P() = point;
@@ -767,9 +800,9 @@ public:
       // compute new vertex on the last (border) edge
       point = (runPos.V()->P() + runPos.VFlip()->P()) / 2.f;
       if (checkSing) {
-        if (onSideA)
+        if (resultCode == PC_SINGSIDEA)
           point = runPos.V()->P();
-        if (onSideB)
+        else if (resultCode == PC_SINGSIDEB)
           point = runPos.VFlip()->P();
       }
       runPos.V()->P() = point;
@@ -843,6 +876,8 @@ public:
     while (!chords.End()) {
       // get the current coord
       chords.GetCurrent(face_edge);
+      resultCode = chords[face_edge].q;
+      assert(resultCode == PC_VOID);
       // construct a pos on the face and edge of the current coord
       pos.Set(&mesh.face[face_edge.first], face_edge.second, mesh.face[face_edge.first].V(face_edge.second));
       // (try to) collapse the polychord
@@ -851,7 +886,7 @@ public:
       chords.Next();
 
       // increment the mark
-      mark++;
+      ++mark;
       if (mark == std::numeric_limits<unsigned long>::max()) {
         chords.ResetMarks();
         mark = 0;
@@ -890,7 +925,7 @@ public:
       // check and find start pos
       resultCode = CheckPolychordFindStartPosition(pos, startPos, false);
       // visit the polychord
-      if (resultCode == PC_SUCCESS || resultCode == PC_SINGBOTH) {
+      if (resultCode == PC_SUCCESS || resultCode == PC_SINGBOTH || resultCode == PC_SINGSIDEA || resultCode == PC_SINGSIDEB) {
         VisitPolychord(mesh, startPos, chords, mark, PC_OTHER);
         // store a new polychord
         if (!loopsOnly)
@@ -898,6 +933,27 @@ public:
         else if (!startPos.IsBorder())
           polychords.push_back(startPos);
       } else {
+        if (resultCode == PC_NOTMANIF) {
+          pos = startPos;
+          VisitPolychord(mesh, pos, chords, mark, resultCode);
+          if (pos.IsManifold() && !pos.IsBorder()) {
+            pos.FlipF();
+            VisitPolychord(mesh, pos, chords, mark, resultCode);
+          }
+        } else if (resultCode == PC_NOTQUAD) {
+          // if not quad, visit all the polychords passing through this coord
+          pos = startPos;
+          do {
+            if (!pos.IsBorder()) {
+              pos.FlipF();
+              VisitPolychord(mesh, pos, chords, mark, resultCode);
+              pos.FlipF();
+            }
+            pos.FlipV();
+            pos.FlipE();
+          } while (pos != startPos);
+          VisitPolychord(mesh, startPos, chords, mark, resultCode);
+        }
         VisitPolychord(mesh, startPos, chords, mark, resultCode);
       }
 
@@ -937,7 +993,7 @@ public:
 
     vcg::face::Pos<FaceType> startPos, runPos;
     PC_ResultCode result = CheckPolychordFindStartPosition(pos, startPos, false);
-    if (result != PC_SUCCESS && result != PC_SINGBOTH)
+    if (result != PC_SUCCESS && result != PC_SINGBOTH && result != PC_SINGSIDEA && result != PC_SINGSIDEB)
       return;
 
     // since every face has an orientation, ensure that the new polychords are inserted on the right of the starting pos
@@ -1013,7 +1069,7 @@ public:
     // allocate and initialize 4 vertices and ffAdj for each new face
     for (FaceIterator fIt = firstAddedFaceIt; fIt != mesh.face.end(); ++fIt) {
       fIt->Alloc(4);
-      for (size_t j = 0; j < 4; j++) {
+      for (size_t j = 0; j < 4; ++j) {
         fIt->FFp(j) = &*fIt;
         fIt->FFi(j) = j;
       }
@@ -1566,7 +1622,7 @@ public:
         // if the vertex is on border increment its valence by 1 (virtually connect it to a dummy vertex)
         jmpPos.Set(startPos.F(), startPos.E(), startPos.V());
         if (jmpPos.FindBorder())
-          valence++;
+          ++valence;
         if (valence != 4)
           singSideB = true;
         // a 2-valence internl vertex cause a polychord to touch itself, producing non-2manifoldness
@@ -1579,7 +1635,7 @@ public:
         // if the vertex is on border increment its valence by 1 (virtually connect it to a dummy vertex)
         jmpPos.Set(startPos.F(), startPos.E(), startPos.V());
         if (jmpPos.FindBorder())
-          valence++;
+          ++valence;
         if (valence != 4)
           singSideA = true;
         // a 2-valence internal vertex cause a polychord to touch itself, producing non-2manifoldness
@@ -1624,6 +1680,13 @@ public:
     if (!polyBorderFound && borderSideA && borderSideB)
       return PC_SINGBOTH;
 
+    // if there are singularities or borders on the side A, remember to keep coordinates on it
+    if (singSideA || borderSideA)
+      return PC_SINGSIDEA;
+    // if there are singularities or borders on the side B, remember to keep coordinates on it
+    if (singSideB || borderSideB)
+      return PC_SINGSIDEB;
+
     return PC_SUCCESS;
   }
 
@@ -1644,14 +1707,13 @@ public:
     vcg::face::Pos<FaceType> tmpPos, runPos = startPos;
     std::pair<size_t, unsigned char> face_edge(std::numeric_limits<size_t>::max(), 0);
 
-    if (runPos.F()->VN() != 4)  // non-quads are not visited
-      return;
-
     // follow the sequence of quads
     do {
       // check manifoldness
       tmpPos = runPos;
       do {
+        if (runPos.F()->VN() != 4)  // non-quads are not visited
+          return;
         if (!tmpPos.IsManifold()) {
           // update current coord
           face_edge.first = vcg::tri::Index(mesh, tmpPos.F());
@@ -1694,6 +1756,7 @@ public:
     vcg::face::JumpingPos<FaceType> jmpPos;
     jmpPos.Set(pos.F(), pos.E(), pos.V());
     do {
+      assert(!jmpPos.FFlip()->IsD());
       if (!jmpPos.IsManifold())
         return true;
       jmpPos.NextFE();
@@ -1737,6 +1800,10 @@ public:
         face_edge.second = (tmpPos.E()+1)%2;
         if (chords[face_edge].mark == mark)
           return true;
+        // this should never hapen:
+        face_edge.second = tmpPos.E()%2;
+        if (chords[face_edge].mark == mark)
+          return true;
       }
       tmpPos = runPos;
       tmpPos.FlipV();
@@ -1747,6 +1814,10 @@ public:
         face_edge.second = (tmpPos.E()+1)%2;
         if (chords[face_edge].mark == mark)
           return true;
+        // this should never hapen:
+        face_edge.second = tmpPos.E()%2;
+        if (chords[face_edge].mark == mark)
+          return true;
       }
       runPos.FlipE();
       runPos.FlipV();
@@ -1756,6 +1827,164 @@ public:
 
     return false;
   }
+
+  /**
+   * @brief WillPolychordBeManifold checks whether a polychord starting at startPos would cause non-manifoldness
+   * if it was collapsed.
+   * @note VisitPolychord() should be called before this method.
+   * @param mesh The input mesh.
+   * @param startPos The starting Pos.
+   * @param chords The vector of chords.
+   * @param mark The current mark, used to identify quads already visited.
+   * @return true if manifoldness remains, false otherwise.
+   */
+  static bool WillPolychordBeManifold(const PolyMeshType &mesh,
+                                      const vcg::face::Pos<FaceType> &startPos,
+                                      PC_Chords &chords,
+                                      const unsigned long mark) {
+    assert(!startPos.IsNull());
+    vcg::face::Pos<FaceType> runPos = startPos;
+    vcg::face::JumpingPos<FaceType> tmpPos;
+    std::pair<size_t, unsigned char> face_edge1(std::numeric_limits<size_t>::max(), 0);
+    std::pair<size_t, unsigned char> face_edge2(std::numeric_limits<size_t>::max(), 0);
+    bool in = true;
+    unsigned int nTraversal = 0;
+
+    // second step: check
+    runPos = startPos;
+    do {
+      face_edge1.first = vcg::tri::Index(mesh, runPos.F());
+      face_edge1.second = runPos.E() % 2;
+      assert(chords[face_edge1].mark == mark);
+      // check one vertex
+      runPos.FlipV();
+      in = true;
+      nTraversal = 0;
+      tmpPos.Set(runPos.F(), runPos.E(), runPos.V());
+      do {
+        if (tmpPos.IsBorder() && in) {
+          in = false;
+          nTraversal++;
+        }
+        // go to next edge
+        tmpPos.NextFE();
+        // check if this face is already visited
+        face_edge1.first = vcg::tri::Index(mesh, tmpPos.F());
+        face_edge1.second = tmpPos.E() % 2;
+        face_edge2.first = vcg::tri::Index(mesh, tmpPos.F());
+        face_edge2.second = (tmpPos.E() + 1) % 2;
+        if (in && chords[face_edge1].mark != mark && chords[face_edge2].mark != mark) {
+          in = false;
+          ++nTraversal;
+        } else if (!in && (chords[face_edge1].mark == mark || chords[face_edge2].mark == mark)) {
+          in = true;
+          ++nTraversal;
+        }
+      } while (tmpPos != runPos);
+      assert(in);
+      assert(nTraversal % 2 == 0);
+      if (nTraversal > 2)
+        return false;
+
+      // check other vertex
+      runPos.FlipV();
+      in = true;
+      nTraversal = 0;
+      tmpPos.Set(runPos.F(), runPos.E(), runPos.V());
+      do {
+        if (tmpPos.IsBorder() && in) {
+          in = false;
+          ++nTraversal;
+        }
+        // go to next edge
+        tmpPos.NextFE();
+        // check if this face is already visited
+        face_edge1.first = vcg::tri::Index(mesh, tmpPos.F());
+        face_edge1.second = tmpPos.E() % 2;
+        face_edge2.first = vcg::tri::Index(mesh, tmpPos.F());
+        face_edge2.second = (tmpPos.E() + 1) % 2;
+        if (in && chords[face_edge1].mark != mark && chords[face_edge2].mark != mark) {
+          in = false;
+          nTraversal++;
+        } else if (!in && (chords[face_edge1].mark == mark || chords[face_edge2].mark == mark)) {
+          in = true;
+          ++nTraversal;
+        }
+      } while (tmpPos != runPos);
+      assert(in);
+      assert(nTraversal % 2 == 0);
+      if (nTraversal > 2)
+        return false;
+
+      runPos.FlipE();
+      runPos.FlipV();
+      runPos.FlipE();
+      runPos.FlipF();
+    } while (runPos != startPos && !runPos.IsBorder());
+    if (runPos.IsBorder()) {
+      // check one vertex
+      runPos.FlipV();
+      in = true;
+      nTraversal = 0;
+      tmpPos.Set(runPos.F(), runPos.E(), runPos.V());
+      do {
+        if (tmpPos.IsBorder() && in) {
+          in = false;
+          ++nTraversal;
+        }
+        // go to next edge
+        tmpPos.NextFE();
+        // check if this face is already visited
+        face_edge1.first = vcg::tri::Index(mesh, tmpPos.F());
+        face_edge1.second = tmpPos.E() % 2;
+        face_edge2.first = vcg::tri::Index(mesh, tmpPos.F());
+        face_edge2.second = (tmpPos.E() + 1) % 2;
+        if (in && chords[face_edge1].mark != mark && chords[face_edge2].mark != mark) {
+          in = false;
+          ++nTraversal;
+        } else if (!in && (chords[face_edge1].mark == mark || chords[face_edge2].mark == mark)) {
+          in = true;
+          ++nTraversal;
+        }
+      } while (tmpPos != runPos);
+      assert(in);
+      assert(nTraversal % 2 == 0);
+      if (nTraversal > 2)
+        return false;
+
+      // check other vertex
+      runPos.FlipV();
+      in = true;
+      nTraversal = 0;
+      tmpPos.Set(runPos.F(), runPos.E(), runPos.V());
+      do {
+        if (tmpPos.IsBorder() && in) {
+          in = false;
+          ++nTraversal;
+        }
+        // go to next edge
+        tmpPos.NextFE();
+        // check if this face is already visited
+        face_edge1.first = vcg::tri::Index(mesh, tmpPos.F());
+        face_edge1.second = tmpPos.E() % 2;
+        face_edge2.first = vcg::tri::Index(mesh, tmpPos.F());
+        face_edge2.second = (tmpPos.E() + 1) % 2;
+        if (in && chords[face_edge1].mark != mark && chords[face_edge2].mark != mark) {
+          in = false;
+          ++nTraversal;
+        } else if (!in && (chords[face_edge1].mark == mark || chords[face_edge2].mark == mark)) {
+          in = true;
+          ++nTraversal;
+        }
+      } while (tmpPos != runPos);
+      assert(in);
+      assert(nTraversal % 2 == 0);
+      if (nTraversal > 2)
+        return false;
+    }
+
+    return true;
+  }
 };
 
 }
