@@ -85,14 +85,11 @@ Edited Comments and GPL license
 
 #ifndef __VCGLIB_MATH_BASE
 #define __VCGLIB_MATH_BASE
-#include <config.h>
+
 #include <float.h>
-#ifdef HAVE_CXX11
 #include <cmath>
 using namespace std;
-#else
-#include <math.h>
-#endif
+
 #include <assert.h>
 #include <limits>
 #include <algorithm>
@@ -191,11 +188,17 @@ inline float   ToRad(const float &a){return float(M_PI)*a/180.0f;}
 inline double  ToDeg(const double &a){return a*180.0/M_PI;}
 inline double  ToRad(const double &a){return M_PI*a/180.0;}
 
+template <typename T>
+int Sgn(T val) {
+    return (T(0) < val) - (val < T(0));
+}
 
 #if defined(_MSC_VER) // Microsoft Visual C++
 template<class T> int IsNAN(T t) {    return _isnan(t) || (!_finite(t)); }
+#elif defined(__MINGW32__) // GCC
+template<class T> int IsNAN(T t) {    return std::isnan(t) || std::isinf(t); }
 #elif defined(__GNUC__) // GCC
-template<class T> int IsNAN(T t) {    return isnan(t) || isinf(t); }
+  template<class T> int IsNAN(T t) {    return std::isnan(t) || std::isinf(t); }
 
 #else // generic
 
