@@ -49,7 +49,7 @@ template <class TT> class EmptyCore: public TT {
 public:
   typedef int FlagType;
   int &Flags()       { assert(0); static int dummyflags(0);  return dummyflags; }
-  int cFlags() const { assert(0); return 0; }
+  int cFlags() const { return 0; }
   static bool HasFlags()   { return false; }
 
   typedef vcg::Point3f CoordType;
@@ -117,10 +117,16 @@ public:
 
   typename TT::EdgePointer &VEp()       { static typename TT::EdgePointer ep=0;  assert(0); return ep; }
   typename TT::EdgePointer cVEp() const { static typename TT::EdgePointer ep=0;  assert(0); return ep; }
-  int &VEi()       { static int z=0; return z;}
-  int cVEi() const { static int z=0; return z;}
+  int &VEi()       { static int z=-1; return z;}
+  int cVEi() const { static int z=-1; return z;}
   static bool HasVEAdjacency()   {   return false; }
-
+  bool IsVEInitialized() const {return static_cast<const typename TT::VertexType *>(this)->cVEi()!=-1;}
+  void VEClear() {
+    if(IsVEInitialized()) {
+      static_cast<typename TT::VertexPointer>(this)->VEp()=0;
+      static_cast<typename TT::VertexPointer>(this)->VEi()=-1;
+    }
+  }
   typename TT::HEdgePointer &VHp()       { static typename TT::HEdgePointer ep=0;  assert(0); return ep; }
   typename TT::HEdgePointer cVHp() const { static typename TT::HEdgePointer ep=0;  assert(0); return ep; }
   int &VHi()       { static int z=0; return z;}
@@ -128,6 +134,7 @@ public:
   static bool HasVHAdjacency()   {   return false; }
 
   typedef float   CurScalarType;
+  typedef float   ScalarTypeCur;
   typedef Point3f CurVecType;
   typedef Point2f CurvatureType;
   float &Kh()       { static float dummy = 0.f; assert(0);return dummy;}
@@ -399,13 +406,13 @@ public: static void Name(std::vector<std::string> & name){name.push_back(std::st
   template <class A, class TT> class Curvature: public TT {
   public:
     typedef Point2<A> CurvatureType;
-    typedef typename CurvatureType::ScalarType ScalarType;
-    const ScalarType &Kh() const { return _hk[0];}
-    const ScalarType &Kg() const { return _hk[1];}
-          ScalarType &Kh()       { return _hk[0];}
-          ScalarType &Kg()       { return _hk[1];}
-          ScalarType cKh() const { return _hk[0];}
-          ScalarType cKg() const { return _hk[1];}
+    typedef typename CurvatureType::ScalarType ScalarTypeCur;
+    const ScalarTypeCur &Kh() const { return _hk[0]; }
+    const ScalarTypeCur &Kg() const { return _hk[1]; }
+          ScalarTypeCur &Kh()       { return _hk[0]; }
+          ScalarTypeCur &Kg()       { return _hk[1]; }
+          ScalarTypeCur cKh() const { return _hk[0]; }
+          ScalarTypeCur cKg() const { return _hk[1]; }
 
           template < class RightValueType>
           void ImportData(const RightValueType  & rVert ) {
@@ -440,18 +447,18 @@ public: static void Name(std::vector<std::string> & name){name.push_back(std::st
 template <class A, class TT> class CurvatureDir: public TT {
 public:
   typedef A CurvatureDirType;
-    typedef typename CurvatureDirType::VecType VecType;
-    typedef typename CurvatureDirType::ScalarType ScalarType;
+    typedef typename CurvatureDirType::VecType CurVecType;
+    typedef typename CurvatureDirType::ScalarType CurScalarType;
 
-    VecType &PD1(){ return _curv.max_dir;}
-    VecType &PD2(){ return _curv.min_dir;}
-    const VecType &cPD1() const {return _curv.max_dir;}
-    const VecType &cPD2() const {return _curv.min_dir;}
+    CurVecType &PD1(){ return _curv.max_dir; }
+    CurVecType &PD2(){ return _curv.min_dir; }
+    const CurVecType &cPD1() const { return _curv.max_dir; }
+    const CurVecType &cPD2() const { return _curv.min_dir; }
 
-    ScalarType &K1(){ return _curv.k1;}
-    ScalarType &K2(){ return _curv.k2;}
-    const ScalarType &cK1() const {return _curv.k1;}
-    const ScalarType &cK2() const {return _curv.k2;}
+    CurScalarType &K1(){ return _curv.k1; }
+    CurScalarType &K2(){ return _curv.k2; }
+    const CurScalarType &cK1() const { return _curv.k1; }
+    const CurScalarType &cK2() const { return _curv.k2; }
     template < class RightValueType>
     void ImportData(const RightValueType  & rVert ) {
       if(rVert.IsCurvatureDirEnabled()) {
