@@ -27,17 +27,15 @@
 #include<vcg/space/index/base.h>
 #include<vcg/space/box3.h>
 #include <vcg/space/index/space_iterators.h>
-#include <config.h>
-
+#include <cstdint>
 //#ifndef WIN32
-#ifdef HAVE_INTTYPES
-#include <inttypes.h>
+
+
 #define __int64 int64_t
 //#define __cdecl 
-#else
-#define __int64 long long
+
 //#define __cdecl
-#endif
+
 
 namespace vcg {
 
