@@ -100,10 +100,6 @@ template<class PolygonType>
 void PolyNormals(const PolygonType &F,
                  std::vector<typename PolygonType::CoordType> &Norms)
 {
-    typedef typename PolygonType::FaceType FaceType;
-    typedef typename PolygonType::CoordType CoordType;
-    typedef typename PolygonType::ScalarType ScalarType;
-
     Norms.clear();
     if (F.VN()<=2) return;
     for (int i=0;i<F.VN();i++)
@@ -126,7 +122,6 @@ typename PolygonType::CoordType PolyBarycenter(const PolygonType &F)
 template<class PolygonType>
 typename PolygonType::ScalarType PolyArea(const PolygonType &F)
 {
-    typedef typename PolygonType::FaceType FaceType;
     typedef typename PolygonType::CoordType CoordType;
     typedef typename PolygonType::ScalarType ScalarType;
 
@@ -159,8 +154,6 @@ typename PolygonType::CoordType PolygonNormal(const PolygonType &F)
 template<class PolygonType>
 typename PolygonType::ScalarType PolyPerimeter(const PolygonType &F)
 {
-    typedef typename PolygonType::FaceType FaceType;
-    typedef typename PolygonType::CoordType CoordType;
     typedef typename PolygonType::ScalarType ScalarType;
 
     ScalarType SumL=0;
@@ -177,11 +170,10 @@ typename PolygonType::ScalarType PolyPerimeter(const PolygonType &F)
 template<class PolygonType>
 typename PolygonType::ScalarType PolyNormDeviation(const PolygonType &F)
 {
-    typedef typename PolygonType::FaceType FaceType;
     typedef typename PolygonType::CoordType CoordType;
     typedef typename PolygonType::ScalarType ScalarType;
 
-    std::vector<typename PolygonType::CoordType> Norms;
+    std::vector<CoordType> Norms;
     PolyNormals(F,Norms);
 
     //calculate the Avg Normal
@@ -191,7 +183,7 @@ typename PolygonType::ScalarType PolyNormDeviation(const PolygonType &F)
 
     AvgNorm.Normalize();
 
-    if (!CheckNormalizedCoords(AvgNorm))return 1;
+    //if (!CheckNormalizedCoords(AvgNorm))return 1;
 
     ScalarType Dev=0;
     for (int i=0;i<Norms.size();i++)
@@ -209,7 +201,6 @@ void PolyAngleDeviation(const PolygonType &F,
                         typename PolygonType::ScalarType &AvgDev,
                         typename PolygonType::ScalarType &MaxDev)
 {
-    typedef typename PolygonType::FaceType FaceType;
     typedef typename PolygonType::CoordType CoordType;
     typedef typename PolygonType::ScalarType ScalarType;
     assert(F.VN()>2);
@@ -246,7 +237,6 @@ void PolyAngleDeviation(const PolygonType &F,
 template<class PolygonType>
 vcg::Plane3<typename PolygonType::ScalarType> PolyFittingPlane(const PolygonType &F)
 {
-    typedef typename PolygonType::FaceType FaceType;
     typedef typename PolygonType::CoordType CoordType;
     typedef typename PolygonType::ScalarType ScalarType;
     vcg::Plane3<ScalarType> BestPL;
@@ -263,7 +253,6 @@ vcg::Plane3<typename PolygonType::ScalarType> PolyFittingPlane(const PolygonType
 template<class PolygonType>
 typename PolygonType::ScalarType PolyFlatness(const PolygonType &F)
 {
-    typedef typename PolygonType::FaceType FaceType;
     typedef typename PolygonType::CoordType CoordType;
     typedef typename PolygonType::ScalarType ScalarType;
 
@@ -293,7 +282,6 @@ template<class PolygonType>
 void PolyPCA(const PolygonType &F,
              typename PolygonType::CoordType PCA[])
 {
-    typedef typename PolygonType::FaceType FaceType;
     typedef typename PolygonType::CoordType CoordType;
     typedef typename PolygonType::ScalarType ScalarType;
 
@@ -361,7 +349,6 @@ template<class PolygonType>
 void PolyScaledPCA(const PolygonType &F,
                    typename PolygonType::CoordType PCA[])
 {
-    typedef typename PolygonType::FaceType FaceType;
     typedef typename PolygonType::CoordType CoordType;
     typedef typename PolygonType::ScalarType ScalarType;
 
@@ -412,9 +399,9 @@ void getBaseTemplatePolygon(int N,
 //described by "Static Aware Grid Shells" by Pietroni et Al.
 template<class PolygonType>
 void GetPolyTemplatePos(const PolygonType &F,
-                        std::vector<typename PolygonType::CoordType> &TemplatePos)
+                        std::vector<typename PolygonType::CoordType> &TemplatePos,
+                        bool force_isotropy=false)
 {
-    typedef typename PolygonType::FaceType FaceType;
     typedef typename PolygonType::CoordType CoordType;
     typedef typename PolygonType::ScalarType ScalarType;
     std::vector<CoordType>  UniformPos,UniformTempl;
@@ -433,6 +420,20 @@ void GetPolyTemplatePos(const PolygonType &F,
     CoordType dirY=PCA[1];
     CoordType dirZ=PCA[2];
 
+    if (force_isotropy)
+    {
+        dirX.Normalize();
+        dirY.Normalize();
+        dirZ.Normalize();
+//        CoordType dirXN=dirX;dirXN.Normalize();
+//        CoordType dirYN=dirY;dirYN.Normalize();
+//        CoordType dirZN=dirZ;dirZN.Normalize();
+
+//        dirX=dirX*0.8+dirXN*0.2;
+//        dirY=dirY*0.8+dirYN*0.2;
+//        dirZ=dirZ*0.8+dirZN*0.2;
+    }
+
     ///set the Rotation matrix
     ToPCA.SetColumn(0,dirX);
     ToPCA.SetColumn(1,dirY);
@@ -455,6 +456,11 @@ void GetPolyTemplatePos(const PolygonType &F,
     ScalarType AreaTemplate=Area(TemplatePos);
     ScalarType AreaUniform=Area(UniformPos);
 
+//    if (TargetArea>0)
+//    {
+//        AreaUniform*=(AreaUniform/TargetArea);
+//    }
+
     ScalarType Scale=sqrt(AreaTemplate/AreaUniform);
 
     for (size_t i=0;i<UniformPos.size();i++)
@@ -472,8 +478,8 @@ void GetPolyTemplatePos(const PolygonType &F,
     ///add displacement along Z
     for (size_t i=0;i<FixPoints.size();i++)
     {
-        FixPoints[i]+=CoordType(0,0,0.01);
-        MovPoints[i]+=CoordType(0,0,0.01);
+        FixPoints[i]+=CoordType(0,0,0.1);
+        MovPoints[i]+=CoordType(0,0,0.1);
     }
     ///add original points
     FixPoints.insert(FixPoints.end(),UniformPos.begin(),UniformPos.end());
@@ -497,16 +503,6 @@ void GetPolyTemplatePos(const PolygonType &F,
         TemplatePos[i]=ToPCAInv*TemplatePos[i];
     }
 
-    //        if (use_fixed_area)
-    //        {
-    //            ScalarType A0=Area(TemplatePos);
-    //            ScalarType A1=FixedArea;
-    //            //ScalarType Scale1=A1/A0;
-    //            for (size_t i=0;i<TemplatePos.size();i++)
-    //                TemplatePos[i]*=A1/A0;
-    //        }
-
-
     for (size_t i=0;i<TemplatePos.size();i++)
         TemplatePos[i]+=Barycenter;
 
@@ -515,14 +511,14 @@ void GetPolyTemplatePos(const PolygonType &F,
 //compute the aspect ratio using the rigidly aligned template polygon as
 //described by "Static Aware Grid Shells" by Pietroni et Al.
 template<class PolygonType>
-typename PolygonType::ScalarType PolyAspectRatio(const PolygonType &F)
+typename PolygonType::ScalarType PolyAspectRatio(const PolygonType &F,
+                                                 bool isotropic=false)
 {
-    typedef typename PolygonType::FaceType FaceType;
     typedef typename PolygonType::CoordType CoordType;
     typedef typename PolygonType::ScalarType ScalarType;
     std::vector<CoordType> TemplatePos;
 
-    GetPolyTemplatePos(F,TemplatePos);
+    GetPolyTemplatePos(F,TemplatePos,isotropic);
 
     ScalarType diff=0;
     assert((int)TemplatePos.size()==F.VN());
